@@ -1,3 +1,5 @@
+__version__ = "0.12.9"
+
 from spb.functions import (
     plot,
     plot_parametric,
@@ -10,11 +12,10 @@ from spb.functions import (
     geometry_plot,
 )
 from spb.plot_data import get_plot_data, smart_plot
+from spb.vectors import vector_plot
+from spb.ccomplex.complex import complex_plot
 
 # from spb.interactive import iplot
-from spb.vectors import vector_plot
-from spb.complex.complex import complex_plot
-
 # from spb.backends.plotgrid import plotgrid
 
 # aliases
@@ -27,28 +28,3 @@ implicit_plot = plot_implicit
 plot_polar = polar_plot
 plot_geometry = geometry_plot
 plot_complex = complex_plot
-
-__all__ = [
-    "plot",
-    "plot_parametric",
-    "plot_contour",
-    "plot3d",
-    "plot3d_parametric_line",
-    "plot3d_parametric_surface",
-    "plot_implicit",
-    "polar_plot",
-    "geometry_plot",
-    "get_plot_data",
-    "smart_plot",
-    "vector_plot",
-    "complex_plot",
-    "parametric_plot",
-    "contour_plot",
-    "p3dpl",
-    "p3dps",
-    "p3d",
-    "implicit_plot",
-    "plot_polar",
-    "plot_geometry",
-    "plot_complex",
-]
