@@ -6,7 +6,7 @@ from spb.series import (
     InteractiveSeries,
     _set_discretization_points
 )
-from spb.complex.complex import _build_series as _build_complex_series
+from spb.ccomplex.complex import _build_series as _build_complex_series
 from spb.vectors import _preprocess, _build_series as _build_vector_series
 from spb.utils import _plot_sympify, _unpack_args
 from spb.defaults import TWO_D_B, THREE_D_B
@@ -48,8 +48,8 @@ class DynamicParam(param.Parameterized):
             softbounds : tuple
                 Tuple of two float (or integer) numbers: (start, end).
             N : int
-                Number of increments in the slider. 
-                (start - end) / N represents the step increment. Default to 40. 
+                Number of increments in the slider.
+                (start - end) / N represents the step increment. Default to 40.
                 Set N=-1 to have unit step increments.
             label : str
                 Label of the slider. Default to None. If None, the string or
